from ..dependencies import *
def append_output(self,key:str,new_content:str)->None:
    """
    appends a line of text to the 'other' section of feedback on the GUI.
    """
    self.window_mgr.update_value(key=key,value=self.window_mgr.get_from_value(key)+'\n\n'+new_content)
    
def get_url(self)->None:
    """
    retrieves the URL entered by the user in the corresponding input field of the UI.
    """
    url = self.window_mgr.get_values()['-URL-']
    if url==None:
        url_list =self.window_mgr.get_values()['-URL_LIST-']
        url = safe_list_return(url_list)
    return url

def get_url_manager(self,url:str=None)->str:
    """
    returns the processed URL. It uses the UrlManager and SafeRequest classes from the abstract_webtools
    module to process and safe-proof the URL.
    """
    url = url or self.get_url()
    url_manager = UrlManager(url=url)    
    return url_manager

def url_event_check(self)->bool:
    """
    Checks if a URL event has been triggered.

    An event is triggered when a user interacts with the URL management interface.
    This function prompts the SafeRequest module to return the HTML content or the
    BeautifulSoup object of the specified URL.

    Returns:
        bool: True if a URL event has been triggered, otherwise False
    """
    if self.event in ['-GET_SOUP-',
                      '-GET_SOURCE_CODE-',
                      '-ADD_URL-']:
        url_manager = self.get_url_manager()
        if self.event in ['-GET_SOUP-',
                          '-GET_SOURCE_CODE-']:
            self.chunk_title=None
            data=self.window_mgr.get_values()['-URL_TEXT-']
            url=None
            if url_manager.url:
                url = url_manager.url
            if self.event=='-GET_SOUP-':
                self.url_chunk_type='SOUP'
                data = url_grabber_component(url=url)
                self.window_mgr.update_value(key='-URL_TEXT-',value=data)
            elif url_manager.url and self.event=='-GET_SOURCE_CODE-':
                url_list =self.window_mgr.get_values()['-URL_LIST-']
                if url_list:
                    url = UrlManager(url=self.window_mgr.get_values()['-URL_LIST-'][0]).url
                    self.chunk_title=url
                    self.window_mgr.update_value(key=text_to_key('-CHUNK_TITLE-',section='url'),value=url)
                request_manager = SafeRequest(url_manager=url_manager)
                if self.event == '-GET_SOURCE_CODE-':
                    self.url_chunk_type='URL'
                    data = request_manager.source_code
            else:
                print(f'url {url} is malformed... aborting...')
        elif self.event == '-ADD_URL-':
            url = url_manager.url
            url_list = make_list(self.window_mgr.get_values()['-URL_LIST-']) or make_list(url)
            if url_list:
                if url not in url_list:
                    url_list.append(url)
            self.window_mgr.update_value(key='-URL_LIST-',args={"values":url_list})
    else:
        return False
    return True




def text_to_key(text, section):
    """ Converts a text description to a GUI key based on the section """
    return f"{section}_{text}".replace(" ", "_").upper()

class InstructionManager_nogui:
    def __init__(self, instructions):
        self.default_instructions = {
            "api_response": {
                "instruction": "Place response to prompt here.",
                "example": "This is the reply to your request.",
                "default": True},
            "additional_responses": {
                "instruction": "Marking 'True' initiates a loop which continues to send the current chunk's prompt until the module returns a false value.",
                "example": "false",
                "default": False},
            # Add other instruction definitions here...
            }
        self.instructions = []
        self.default_instructions.update({key:{"instruction":instructions[key] if not isinstance(instructions[key],dict) else instructions[key].get('instruction',instructions[key]),"example":instructions[key] if not isinstance(instructions[key],dict) else instructions.get("example",""),"default":True} for key in instructions})
        self.get_default_instructions()
        self.make_instructions_reference()
        
    def get_default_instructions(self):
        """ Loads the default instructions based on what's set as default """
        self.instructions = [{key: self.default_instructions[key]} for key in self.default_instructions if self.default_instructions[key]["default"]]
        return self.instructions

    def update_instructions(self):
        """ Updates the instructions dynamically from the GUI values """
        bools = {}
        instructions = {}
        for key, value in self.default_instructions.items():
            bools[key] = self.window_mgr.get_values().get(text_to_key(text=key, section="BOOL"))
            instructions[key] = self.window_mgr.get_values().get(text_to_key(text=key, section="TEXT"))
        self.update_instructions_bools(**bools)
        self.update_instructions_text_values(**instructions)

    def update_instructions_bools(self, **bools):
        """ Updates boolean settings for instructions based on GUI input """
        for key, value in bools.items():
            if value is not None:
                self.instructions[key]["default"] = bool(value)

    def update_instructions_text_values(self, **text_values):
        """ Updates text settings for instructions based on GUI input """
        for key, value in text_values.items():
            if value is not None:
                self.instructions[key]["instruction"] = value

    def make_instructions_reference(self):
        """ Constructs a reference dictionary for the GUI to use """
        instructions_bools = {key: instr["default"] for key, instr in self.instructions[0].items()}
        instructions_text = {key: instr["instruction"] for key, instr in self.instructions[0].items()}
        self.instructions={
            "instructions_bools": instructions_bools,
            "instructions_text_values": instructions_text,
            "text": self.get_instructions_text(),
        }

    def get_instructions_text(self):
        """ Formats the instructions for display in the GUI """
        instructions = "your response is expected to be in JSON format with the keys as follows:\n\n"
        i = 0
        for key, value in self.instructions[0].items():
            instructions += f"{i}) {key} - {value.get('instruction')}\n"
            i += 1
        return instructions



from .response_section import *
from .output_display import *
from .output_file_utils import *
from .output_response import *
from .progress_display import *
from .query_events import *
from .response_section import *
